#!/bin/bash -e
#
# $Id: build-updates.sh 17666 2020-08-01 05:09:44Z NiLuJe $
#

HACKDIR="Fonts"
HACKNAME="linkfonts"
PKGNAME="${HACKNAME##*link}"
PKGVER="5.16.N"

# Setup KindleTool packaging metadata flags to avoid cluttering the invocations
PKGREV="$(svnversion -c .. | awk '{print $NF}' FS=':' | tr -d 'P')"
KT_PM_FLAGS=( "-xPackageName=${HACKDIR}" "-xPackageVersion=${PKGVER}-r${PKGREV}" "-xPackageAuthor=NiLuJe" "-xPackageMaintainer=NiLuJe" "-X" )

# Do we have kindletool (https://github.com/NiLuJe/KindleTool) in $PATH?
if kindletool help &>/dev/null ; then
	USE_KINDLETOOL="true"
fi

# We also need GNU tar
if [[ "$(uname -s)" == "Darwin" ]] ; then
	TAR_BIN="gtar"
else
	TAR_BIN="tar"
fi
if ! ${TAR_BIN} --version | grep -q "GNU tar" ; then
	echo "You need GNU tar to build this package."
	exit 1
fi

# Force use of the python tool
#USE_KINDLETOOL="false"

# Pickup our common stuff... We leave it in our staging wd so it ends up in the source package.
if [[ ! -d "../../Common" ]] ; then
	echo "The tree isn't checked out in full, missing the Common directory..."
	exit 1
fi
# LibOTAUtils
ln -f ../../Common/lib/libotautils ./libotautils
# XZ Utils
ln -f ../../Common/bin/xzdec ./xzdec
# LibKH
for common_lib in libkh ; do
	ln -f ../../Common/lib/${common_lib} ../src/${HACKNAME}/bin/${common_lib}
done
# USB Watchdog
for common_bin in usb-watchdog usb-watchdog-helper ; do
	ln -f ../../Common/bin/${common_bin} ../src/${HACKNAME}/bin/${common_bin}
done
# FBInk
ln -f ../../Common/bin/fbink ../src/${HACKNAME}/bin/fbink

# Setup xz multi-threading...
export XZ_DEFAULTS="-T 0"

# OTAv2 MRPI only packages or not?
if [[ "${NO_MRPI_ONLY_PACKAGES}" == "true" ]] ; then
	# K2 (Don't bundle extra K3 only fonts & utils (my current fc-scan binary doesn't run on K2 devices anyway), to save a bit of space)
	KINDLE_MODELS="k2 k2i dx dxi dxg"

	# Archive custom directory
	${TAR_BIN} --hard-dereference --owner root --group root --exclude-vcs --exclude="*Caecilia_LT_*.ttf" --exclude="*Helvetica_LT_*.ttf" --exclude="*KindleBlackbox*.ttf" --exclude="*Kindle_*Symbol.ttf" --exclude="code2000.ttf" --exclude="HangulGothMTC_E*.ttf" --exclude="HeiSeiMaruGoth213_E*.ttf" --exclude="MHeiM18030_E*.ttf" --exclude="MHeiM-Big5HKSCS_E*.ttf" --exclude="CJK.ttf" --exclude="I18N.ttf" --exclude="fc-scan" --exclude="libexpat.so.1" --exclude="libfontconfig.so.1" --exclude="linkfonts/lib/spr" -cvJf ${HACKNAME}.tar.xz ../src/${HACKNAME} ../src/extensions

	for model in ${KINDLE_MODELS} ; do
		# Prepare our files for this specific kindle model...
		ARCH=${PKGNAME}_${PKGVER}_${model}

		if [[ "$USE_KINDLETOOL" == "true" ]] ; then
			# Install
			kindletool create ota -d ${model} libotautils install.sh ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec Update_${ARCH}_install.bin
			# Uninstall
			kindletool create ota -d ${model} libotautils uninstall.sh Update_${ARCH}_uninstall.bin
		else
			# Build install update
			./kindle_update_tool.py m --${model} --sign ${ARCH}_install libotautils install.sh ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec

			# Build uninstall update
			./kindle_update_tool.py m --${model} --sign ${ARCH}_uninstall libotautils uninstall.sh
		fi
	done

	# Remove custom directory archive
	rm -f ${HACKNAME}.tar.xz
fi

# K3
KINDLE_MODELS="k3g k3w k3gb"

# Archive custom directory
${TAR_BIN} --hard-dereference --owner root --group root --exclude-vcs --exclude="*Caecilia_LT_*.ttf" --exclude="*Helvetica_LT_*.ttf" --exclude="*KindleBlackbox*.ttf" --exclude="*Kindle_*Symbol.ttf" --exclude="code2000.ttf" --exclude="HangulGothMTC_E*.ttf" --exclude="HeiSeiMaruGoth213_E*.ttf" --exclude="MHeiM18030_E*.ttf" --exclude="MHeiM-Big5HKSCS_E*.ttf" --exclude="linkfonts/lib/spr" -cvJf ${HACKNAME}.tar.xz ../src/${HACKNAME} ../src/extensions

# OTAv2 MRPI only packages or not?
if [[ "${NO_MRPI_ONLY_PACKAGES}" == "true" ]] ; then
	for model in ${KINDLE_MODELS} ; do
		# Prepare our files for this specific kindle model...
		ARCH=${PKGNAME}_${PKGVER}_${model}

		if [[ "$USE_KINDLETOOL" == "true" ]] ; then
			# Install
			kindletool create ota -d ${model} libotautils install.sh ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec Update_${ARCH}_install.bin
			# Uninstall
			kindletool create ota -d ${model} libotautils uninstall.sh Update_${ARCH}_uninstall.bin
		else
			# Build install update
			./kindle_update_tool.py m --${model} --sign ${ARCH}_install libotautils install.sh ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec

			# Build uninstall update
			./kindle_update_tool.py m --${model} --sign ${ARCH}_uninstall libotautils uninstall.sh
		fi
	done
else
	if [[ "$USE_KINDLETOOL" == "true" ]] ; then
		ARCH="${PKGNAME}_${PKGVER}_k2_dx_k3"
		# Install
		kindletool create ota2 "${KT_PM_FLAGS[@]}" -d legacy libotautils install.sh ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec Update_${ARCH}_install.bin
		# Uninstall
		kindletool create ota2 "${KT_PM_FLAGS[@]}" -d legacy -d kindle4 libotautils uninstall.sh Update_${PKGNAME}_${PKGVER}_uninstall.bin
	fi
fi

# Remove custom directory archive
rm -f ${HACKNAME}.tar.xz

# K4
if [[ "$USE_KINDLETOOL" == "true" ]] ; then
	## Go away if we don't have the Touch_hacks tree checked out...
	if [[ ! -d "../../../Touch_Hacks" ]] ; then
		echo "Skipping K4 build, we don't have the proper binaries on hand (hint: check out the Touch_Hacks tree, too)"
	else
		# We really, really don't want to silently forget those...
		if [[ ! -d "../../../K4_Hacks" ]] ; then
			echo "We're missing the K4-only binaries (from the K4_Hacks tree)"
			exit 1
		fi
		# NOTE: This isn't particularly pretty, and we end up bundling only one version of the binaries in the zip files, but at least the .bin files end up with the correct set of binaries ;)
		# K4/K5 binaries...
		KINDLE_MODEL_BINARIES="bin/fc-scan bin/shlock lib/autohint/libfreetype.so lib/bci/libfreetype.so lib/light/libfreetype.so lib/libharfbuzz.so.0 lib/libexpat.so.1 lib/libfontconfig.so.1 lib/libz.so.1 lib/libfreetype.so.6 lib/libuuid.so.1"

		# Archive custom directory
		${TAR_BIN} --hard-dereference --owner root --group root --exclude-vcs --exclude="*Caecilia_LT_*.ttf" --exclude="*Helvetica_LT_*.ttf" --exclude="*KindleBlackbox*.ttf" --exclude="*Kindle_*Symbol.ttf" --exclude="code2000.ttf" --exclude="HangulGothMTC_E*.ttf" --exclude="HeiSeiMaruGoth213_E*.ttf" --exclude="MHeiM18030_E*.ttf" --exclude="MHeiM-Big5HKSCS_E*.ttf" --exclude="linkfonts/lib/spr" -cvf ${HACKNAME}.tar ../src/${HACKNAME} ../src/extensions
		# Delete K3 binaries
		for my_bin in ${KINDLE_MODEL_BINARIES} ; do
			${TAR_BIN} --delete -vf ${HACKNAME}.tar src/${HACKNAME}/${my_bin}
		done
		# Append K4 binaries
		for my_bin in ${KINDLE_MODEL_BINARIES} ; do
			# NOTE: If we have a K4-specific version of a specific binary, use it instead of the classic K5 one...
			#       This is to handle the weird crashes with FT overrides when FT is built w/ GCC 5, so we use a set built w/ GCC 4.9 instead...
			if [[ -f ../../../K4_Hacks/${HACKDIR}/src/${HACKNAME}/${my_bin} ]] ; then
				${TAR_BIN} --hard-dereference --owner root --group root --transform "s,^K4_Hacks/${HACKDIR}/,,S" --show-transformed-names -rvf ${HACKNAME}.tar ../../../K4_Hacks/${HACKDIR}/src/${HACKNAME}/${my_bin}
			else
				${TAR_BIN} --hard-dereference --owner root --group root --transform "s,^Touch_Hacks/${HACKDIR}/,,S" --show-transformed-names -rvf ${HACKNAME}.tar ../../../Touch_Hacks/${HACKDIR}/src/${HACKNAME}/${my_bin}
			fi
		done
		# Do the same for FBInk
		${TAR_BIN} --delete -vf ${HACKNAME}.tar src/${HACKNAME}/bin/fbink
		${TAR_BIN} --hard-dereference --owner root --group root --transform "s,^Touch_Hacks/Common/,src/${HACKNAME}/,S" --show-transformed-names -rvf ${HACKNAME}.tar ../../../Touch_Hacks/Common/bin/fbink
		# xz it...
		xz ${HACKNAME}.tar

		# Speaking of, we need our own xzdec binary, too!
		ln -f ../../../Touch_Hacks/Common/bin/xzdec ./xzdec

		# Prepare our files for this specific kindle models...
		ARCH=${PKGNAME}_${PKGVER}_k4

		# Install
		kindletool create ota2 "${KT_PM_FLAGS[@]}" -d kindle4 libotautils install.sh ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec Update_${ARCH}_install.bin
		# Uninstall has already been taken care of if we're building unified packages :).
		if [[ "${NO_MRPI_ONLY_PACKAGES}" == "true" ]] ; then
			# Uninstall
			kindletool create ota2 "${KT_PM_FLAGS[@]}" -d kindle4 libotautils uninstall.sh Update_${ARCH}_uninstall.bin
		fi

		# Remove custom directory archive
		rm -f ${HACKNAME}.tar.xz
	fi
fi

# Cleanup
rm -f xzdec

# Move our updates :)
mv -f *.bin ../
