#!/bin/sh
#
# $Id: install.sh 15011 2018-06-02 16:58:21Z NiLuJe $
#

# Pull libOTAUtils for logging & progress handling
[ -f ./libotautils ] && source ./libotautils


# Hack specific config (name and when to start/stop)
HACKNAME="linkfonts"
SLEVEL="62"
KLEVEL="07"

otautils_update_progressbar

# Remove our deprecated content
# From v3.1.N
logmsg "I" "install" "" "removing deprecated symlinks (<= v3.1.N)"
[ -h /etc/rcS.d/S73${HACKNAME} ] && rm -f /etc/rcS.d/S73${HACKNAME}

otautils_update_progressbar

# From v3.2.N
logmsg "I" "install" "" "removing deprecated symlinks (v3.2.N)"
[ -h /etc/rc6.d/K7${HACKNAME} ] && rm -f /etc/rc6.d/K7${HACKNAME}
[ -h /etc/rc0.d/K7${HACKNAME} ] && rm -f /etc/rc0.d/K7${HACKNAME}
[ -h /etc/rc3.d/K7${HACKNAME} ] && rm -f /etc/rc3.d/K7${HACKNAME}

otautils_update_progressbar

# From v3.8.N
logmsg "I" "install" "" "removing deprecated config files (<= v3.8.N)"
HACK_CFGLIST="font.properties-2 font.properties-3 font.properties-3-nocondensed font.properties fonts.conf local.conf local.conf.template netfront.ini prettyversion.txt"
for hack_cfg in ${HACK_CFGLIST} ; do
    [ -f /mnt/us/${HACKNAME}/bin/${hack_cfg} ] && rm -f /mnt/us/${HACKNAME}/bin/${hack_cfg}
done

otautils_update_progressbar

# From v4.1.N
logmsg "I" "install" "" "removing deprecated config file (v4.1.N)"
if [ -f /etc/kdb/system/daemon/pmond/browserd/env ] ; then
    # Only remove it if it's actually our own!
    if grep -q '^LD_LIBRARY_PATH=/mnt/us/linkfonts/lib$' /etc/kdb/system/daemon/pmond/browserd/env ; then
        logmsg "I" "install" "" "custom deprecated config file found, removing"
        rm -f /etc/kdb/system/daemon/pmond/browserd/env
    fi
    # Also remove blank files
    if [ "$( stat -c %s /etc/kdb/system/daemon/pmond/browserd/env )" == "0" ] ; then
        logmsg "I" "install" "" "buggy deprecated config file found, removing"
        rm -f /etc/kdb/system/daemon/pmond/browserd/env
    fi
fi
# And remove the useless copy we had in linkfonts/etc
[ -f /mnt/us/${HACKNAME}/etc/env ] && rm -f /mnt/us/${HACKNAME}/etc/env

otautils_update_progressbar

# From v4.2.N
logmsg "I" "install" "" "removing deprecated config file (v4.2.N)"
if [ -f /etc/kdb/system/daemon/pmond/browserd/env ] ; then
    # Only remove it if it's actually our own!
    if grep -q '^OH_HAI_LINKFONTS=true$' /etc/kdb/system/daemon/pmond/browserd/env ; then
        logmsg "I" "install" "" "custom config file found, removing"
        rm -f /etc/kdb/system/daemon/pmond/browserd/env
    fi
fi

otautils_update_progressbar

# From v4.2
# Remove browser workaround file/directory
if [ -d /var/local/${HACKNAME} ] ; then
    # Be a wee bit cautious, in case something goes alwfully wrong...
    if [ -f /var/local/${HACKNAME}/env ] ; then
        logmsg "I" "install" "" "removing deprecated browser workaround file"
        rm -rf /var/local/${HACKNAME}/env
    fi
    # Only remove the directory if it's really empty...
    if [ "x$( ls -A /var/local/${HACKNAME} 2> /dev/null )" == x  ] ; then
        logmsg "I" "install" "" "removing empty browser workaround directory"
        rm -rf /var/local/${HACKNAME}
    fi
fi

otautils_update_progressbar

# From v4.1.N
logmsg "I" "install" "" "removing deprecated symlink (<= v4.1.N)"
[ -h /etc/rc5.d/S72${HACKNAME} ] && rm -f /etc/rc5.d/S72${HACKNAME}

otautils_update_progressbar

# From v5.8.N
logmsg "I" "install" "" "removing deprecated bci_new ft lib (<= v5.8.N)"
[ -d /mnt/us/${HACKNAME}/lib/bci_new ] && rm -rf /mnt/us/${HACKNAME}/lib/bci_new

otautils_update_progressbar

# From v5.8.N
logmsg "I" "install" "" "migrating bci_new config"
[ -f /mnt/us/${HACKNAME}/bci_new ] && mv -f /mnt/us/${HACKNAME}/bci_new /mnt/us/${HACKNAME}/bci

otautils_update_progressbar

# From v5.16.N
logmsg "I" "install" "" "removing deprecated ft lib extra copy"
[ -f "/mnt/us/${HACKNAME}/lib/bci/libfreetype.so.6" ] && rm -rf "/mnt/us/${HACKNAME}/lib/bci/libfreetype.so.6"

otautils_update_progressbar

# Install our hack's custom content
# But keep the user's custom content...
if [ -d /mnt/us/${HACKNAME} ] ; then
    logmsg "I" "install" "" "our custom directory already exists, checking if we have custom content to preserve"
    # Custom Fonts
    # Loop through all of our custom fonts
    for font_type in Serif Sans Mono ; do
        for font_style in Regular Bold Italic BoldItalic ; do
            current_font="${font_type}_${font_style}.ttf"
            # Select the correct expected md5sum for our current font
            case "${current_font}" in
                "Serif_Regular.ttf" )
                    font_expected_md5="e18165ce2b700b1b4daca82e276fcac5 bfb2f44a7c1deba39f7f4d39bff18eeb"
                ;;
                "Serif_Bold.ttf" )
                    font_expected_md5="3b6c3214b028debd9f175b63f71508e0 849a92990a80cbb665bfc74fd03743bd"
                ;;
                "Serif_Italic.ttf" )
                    font_expected_md5="1262ad9a9f243ef2e7c80c0bb5f0060d a2e7305a0ba8bb7091124f4cd1485fc9"
                ;;
                "Serif_BoldItalic.ttf" )
                    font_expected_md5="1f5dc2bf62705a7d09533b1538c9c173 a062025df92affc1331a05b7c07793fc"
                ;;
                "Sans_Regular.ttf" )
                    font_expected_md5="9e94decf013d3e2c9adcc0b97cc5ce44 9d83fb20700a3a7c45dc9acd64ab121e"
                ;;
                "Sans_Bold.ttf" )
                    font_expected_md5="3c0f650c88f1dd66abf4e2d86b2789b8 2afdf28d5cdd079b41968cdabf1b469e"
                ;;
                "Sans_Italic.ttf" )
                    font_expected_md5="ee14498581142ad399d0b7bfb1d70e3f 6f065779222f5f541356ae7c75dc93fd 5f2a66bd428ec6b298687d9337cb4395"
                ;;
                "Sans_BoldItalic.ttf" )
                    font_expected_md5="a853c46d69c39abc45d15601c7c8b5c1 b71566eb8206848f8314ec8ac5a77e36 139e3a3f44fae921c174554f3754de04"
                ;;
                "Mono_Regular.ttf" )
                    font_expected_md5="0fa23684c88737952788b01bba227ea9 84d533cd5ab154ad1cea6f3a6b46236d 3909da5d4d4df4dea6426e54f622cb28"
                ;;
                "Mono_Bold.ttf" )
                    font_expected_md5="26dd9aa93361477f10b4f949a0a76a61 9a0c0532173736ac68650aad297b8e1d 73dbc55a8e9e588e338f467d8998e358"
                ;;
                "Mono_Italic.ttf" )
                    font_expected_md5="f5b435e042123f193035256df7f70d4a 585945d85b03549f3a4c4cc661f57617 7ac5d510e1c436a4f8cb8e1f66bed302"
                ;;
                "Mono_BoldItalic.ttf" )
                    font_expected_md5="44f2c073f5676fcf0c1748e9e28f2b03 ee3341fd8bdb3e6f157b088efca1d06b 4345faca71e1f388182af8e9621026b8"
                ;;
                * )
                    logmsg "I" "install" "" "huh. unknown custom font for md5 checks. that shouldn't happen"
                ;;
            esac
            # And perform the actual check
            font_md5_match="false"
            # Check if it exists at all...
            if [ -f /mnt/us/${HACKNAME}/fonts/${current_font} ] ; then
                font_current_md5=$( md5sum /mnt/us/${HACKNAME}/fonts/${current_font} | awk '{ print $1; }' )
                for cur_exp_md5 in ${font_expected_md5} ; do
                    if [ "${font_current_md5}" == "${cur_exp_md5}" ] ; then
                        font_md5_match="true"
                    fi
                done
                if [ "${font_md5_match}" != "true" ] ; then
                    HACK_EXCLUDE="${HACK_EXCLUDE} ${HACKNAME}/fonts/${current_font}"
                    logmsg "I" "install" "" "found custom font ${current_font}, excluding from archive"
                fi
            else
                logmsg "I" "install" "" "custom font ${current_font} was missing, it'll be installed"
            fi
        done
    done
    # And also loop through the new K3 fonts
    for font in CJK I18N ; do
        current_font="${font}.ttf"
        # Select the correct expected md5sum for our current font
        case "${current_font}" in
            "CJK.ttf" )
                font_expected_md5="8e286e1a68c8baa259c7cc006bb84056 939889bec9cda51381a5065df3a3bbfd 5931b08bba8127286c06ea0061df0088 4caeadd734f4be9973163bdf02ea6cf5"
            ;;
            "I18N.ttf" )
                font_expected_md5="eccb7a74720fc377b60d6b2110530fd9 bd298363f255b13c67b55bb1ccb4b581"
            ;;
            * )
                logmsg "I" "install" "" "huh. unknown custom font for md5 checks. that shouldn't happen"
            ;;
        esac
        # And perform the actual check
        font_md5_match="false"
        # Check if it exists at all...
        if [ -f /mnt/us/${HACKNAME}/fonts/${current_font} ] ; then
            font_current_md5=$( md5sum /mnt/us/${HACKNAME}/fonts/${current_font} | awk '{ print $1; }' )
            for cur_exp_md5 in ${font_expected_md5} ; do
                if [ "${font_current_md5}" == "${cur_exp_md5}" ] ; then
                    font_md5_match="true"
                fi
            done
            if [ "${font_md5_match}" != "true" ] ; then
                HACK_EXCLUDE="${HACK_EXCLUDE} ${HACKNAME}/fonts/${current_font}"
                logmsg "I" "install" "" "found custom font ${current_font}, excluding from archive"
            fi
        else
            logmsg "I" "install" "" "custom font ${current_font} was missing, it'll be installed if you're on FW 3.x"
        fi
    done
    # Original fonts
    ORIG_FONT_LIST="Caecilia_LT_65_Medium Caecilia_LT_66_Medium_Italic Caecilia_LT_75_Bold Caecilia_LT_76_Bold_Italic Helvetica_LT_65_Medium Helvetica_LT_66_Medium_Italic Helvetica_LT_75_Bold Helvetica_LT_76_Bold_Italic KindleBlackboxBoldItalic KindleBlackboxBold KindleBlackboxItalic KindleBlackboxRegular Kindle_MonospacedSymbol Kindle_Symbol"
    # And the new K3 fonts
    ORIG_FONT_LIST="${ORIG_FONT_LIST} Caecilia_LT_67_Cond_Medium Caecilia_LT_68_Cond_Medium_Italic Caecilia_LT_77_Cond_Bold Caecilia_LT_78_Cond_Bold_Italic code2000 HangulGothMTC_E_Bold HangulGothMTC_E HeiSeiMaruGoth213_E_Bold HeiSeiMaruGoth213_E MHeiM18030_E_Bold MHeiM18030_E MHeiM-Big5HKSCS_E_Bold MHeiM-Big5HKSCS_E"
    # Loop through our copy of all the original fonts to remove them, in case they've been updated by an official fw update. The hack will reinstall them on the next run.
    for original_font in ${ORIG_FONT_LIST} ; do
        if [ -f /mnt/us/${HACKNAME}/fonts/${original_font}.ttf ] ; then
            logmsg "I" "install" "" "removing possibly outdated hack copy of the original font ${original_font}"
            rm -f /mnt/us/${HACKNAME}/fonts/${original_font}.ttf
        fi
    done
    # Cleanup the deprecated default libfreetype backup
    if [ -f /mnt/us/${HACKNAME}/lib/libfreetype.so.6 ] ; then
        logmsg "I" "install" "" "removing deprecated copy of the original freetype lib"
        rm -f /mnt/us/${HACKNAME}/lib/libfreetype.so.6
    fi
    # If we're using the browser/KF8 tweaks, don't disable them ;)
    if [ ! -f /mnt/us/${HACKNAME}/nobrowser ] ; then
        HACK_EXCLUDE="${HACK_EXCLUDE} ${HACKNAME}/nobrowser"
        logmsg "I" "install" "" "using the browser/kf8 tweak, excluding from archive"
    fi
    # Autoreboot
    if [ ! -f /mnt/us/${HACKNAME}/autoreboot ] ; then
        # If we disabled autoreboot, don't re-enable it...
        HACK_EXCLUDE="${HACK_EXCLUDE} ${HACKNAME}/autoreboot"
        logmsg "I" "install" "" "keep autoreboot status (off)"
    fi
fi

otautils_update_progressbar

# Okay, now we can extract it. Since busybox's tar is very limited, we have to use a tmp directory to perform our filtering
logmsg "I" "install" "" "installing custom directory"
# Make sure our xzdec binary is executable first...
chmod +x ./xzdec
./xzdec ${HACKNAME}.tar.xz | tar -xvf -
# Do check if that went well (FIXME: Should we grow the /var tmpfs (which we're writing to), like official updates? We don't use *that* much storage space...)
_RET=$?
if [ ${_RET} -ne 0 ] ; then
    logmsg "C" "install" "code=${_RET}" "failed to extract custom directory in tmp location"
    return 1
fi

cd src
# And now we filter the content to preserve user's custom content
for custom_file in ${HACK_EXCLUDE} ; do
    if [ -f "./${custom_file}" ] ; then
        logmsg "I" "install" "" "preserving custom content (${custom_file})"
        rm -f "./${custom_file}"
    fi
done
# Finally, unleash our filtered dir on the live userstore
cp -af . /mnt/us/
_RET=$?
if [ ${_RET} -ne 0 ] ; then
    logmsg "C" "install" "code=${_RET}" "failure to update userstore with custom directory"
    return 1
fi
cd - >/dev/null
rm -rf src

otautils_update_progressbar

# Install our hack's init script
logmsg "I" "install" "" "installing init script"
cp -f ${HACKNAME}-init /etc/init.d/${HACKNAME}

otautils_update_progressbar

# Make it executable
logmsg "I" "install" "" "chmoding init script"
[ -x /etc/init.d/${HACKNAME} ] || chmod +x /etc/init.d/${HACKNAME}

otautils_update_progressbar

# Make it start at boot (rc5), after dbus and before pmond and the framework
logmsg "I" "install" "" "creating boot runlevel symlink"
[ -h /etc/rc5.d/S${SLEVEL}${HACKNAME} ] || ln -fs /etc/init.d/${HACKNAME} /etc/rc5.d/S${SLEVEL}${HACKNAME}

otautils_update_progressbar

# Make it stop at reboot (rc6), after the framework and before userstore
logmsg "I" "install" "" "creating reboot runlevel symlink"
[ -h /etc/rc6.d/K${KLEVEL}${HACKNAME} ] || ln -fs /etc/init.d/${HACKNAME} /etc/rc6.d/K${KLEVEL}${HACKNAME}

otautils_update_progressbar

# Make it stop at shutdown (rc0), after the framework and before userstore
logmsg "I" "install" "" "creating shutdown runlevel symlink"
[ -h /etc/rc0.d/K${KLEVEL}${HACKNAME} ] || ln -fs /etc/init.d/${HACKNAME} /etc/rc0.d/K${KLEVEL}${HACKNAME}

otautils_update_progressbar

# Make it stop when updating (rc3), after the framework and before the updater
logmsg "I" "install" "" "creating update runlevel symlink"
[ -h /etc/rc3.d/K${KLEVEL}${HACKNAME} ] || ln -fs /etc/init.d/${HACKNAME} /etc/rc3.d/K${KLEVEL}${HACKNAME}

otautils_update_progressbar

# Cleanup
logmsg "I" "install" "" "cleaning up"
rm -f ${HACKNAME}-init ${HACKNAME}.tar.xz xzdec

otautils_update_progressbar

logmsg "I" "install" "" "done"

otautils_update_progressbar

return 0
