#!/bin/sh
#
# $Id: uninstall.sh 17950 2020-11-21 22:04:08Z NiLuJe $
#

# Pull libOTAUtils for logging & progress handling
[ -f ./libotautils ] && source ./libotautils


# Hack specific config (name and when to start/stop)
HACKNAME="linkfonts"
SLEVEL="62"
KLEVEL="07"

otautils_update_progressbar

# From v3.1.N
logmsg "I" "uninstall" "" "removing boot symlink from <= v3.1.N"
[ -h /etc/rcS.d/S73${HACKNAME} ] && rm -f /etc/rcS.d/S73${HACKNAME}

otautils_update_progressbar

# From v3.2.N
logmsg "I" "uninstall" "" "removing messed up symlinks from v3.2.N"
[ -h /etc/rc6.d/K7${HACKNAME} ] && rm -f /etc/rc6.d/K7${HACKNAME}
[ -h /etc/rc0.d/K7${HACKNAME} ] && rm -f /etc/rc0.d/K7${HACKNAME}
[ -h /etc/rc3.d/K7${HACKNAME} ] && rm -f /etc/rc3.d/K7${HACKNAME}

otautils_update_progressbar

# From v4.1.N
logmsg "I" "uninstall" "" "removing boot symlink from <= v4.1.N"
[ -h /etc/rc5.d/S72${HACKNAME} ] && rm -f /etc/rc5.d/S72${HACKNAME}

otautils_update_progressbar

# From v3.3.N
# Boot symlink
logmsg "I" "uninstall" "" "removing boot symlink"
[ -h /etc/rc5.d/S${SLEVEL}${HACKNAME} ] && rm -f /etc/rc5.d/S${SLEVEL}${HACKNAME}

otautils_update_progressbar

# Reboot symlink
logmsg "I" "uninstall" "" "removing reboot symlink"
[ -h /etc/rc6.d/K${KLEVEL}${HACKNAME} ] && rm -f /etc/rc6.d/K${KLEVEL}${HACKNAME}

otautils_update_progressbar

# Shutdown symlink
logmsg "I" "uninstall" "" "removing shutdown symlink"
[ -h /etc/rc0.d/K${KLEVEL}${HACKNAME} ] && rm -f /etc/rc0.d/K${KLEVEL}${HACKNAME}

otautils_update_progressbar

# Updater symlink
logmsg "I" "uninstall" "" "removing update symlink"
[ -h /etc/rc3.d/K${KLEVEL}${HACKNAME} ] && rm -f /etc/rc3.d/K${KLEVEL}${HACKNAME}

otautils_update_progressbar

# Remove our hack's init script
logmsg "I" "uninstall" "" "removing init script"
[ -f /etc/init.d/${HACKNAME} ] && rm -f /etc/init.d/${HACKNAME}

otautils_update_progressbar

# From v4.1.N
logmsg "I" "uninstall" "" "removing deprecated config file from v4.1.N"
if [ -f /etc/kdb/system/daemon/pmond/browserd/env ] ; then
    # Only remove it if it's actually our own!
    if grep -q '^LD_LIBRARY_PATH=/mnt/us/linkfonts/lib$' /etc/kdb/system/daemon/pmond/browserd/env ; then
        logmsg "I" "uninstall" "" "custom deprecated config file found, removing"
        rm -f /etc/kdb/system/daemon/pmond/browserd/env
    fi
    # Also remove blank files
    if [ "$( stat -c %s /etc/kdb/system/daemon/pmond/browserd/env )" == "0" ] ; then
        logmsg "I" "uninstall" "" "buggy deprecated config file found, removing"
        rm -f /etc/kdb/system/daemon/pmond/browserd/env
    fi
fi

otautils_update_progressbar

# From v4.2.N
logmsg "I" "uninstall" "" "removing config file from v4.2.N"
if [ -f /etc/kdb/system/daemon/pmond/browserd/env ] ; then
    # Only remove it if it's actually our own!
    if grep -q '^OH_HAI_LINKFONTS=true$' /etc/kdb/system/daemon/pmond/browserd/env ; then
        logmsg "I" "uninstall" "" "custom config file found, removing"
        rm -f /etc/kdb/system/daemon/pmond/browserd/env
    fi
fi

otautils_update_progressbar

# From v3.8.N
# Remove custom fontconfig cache (FW 3.x only)
if [ -d /var/local/cache/fontconfig ] ; then
    logmsg "I" "uninstall" "" "removing custom fontconfig cache directory"
    rm -rf /var/local/cache/fontconfig
fi

otautils_update_progressbar

# From v4.2
# Remove browser workaround file/directory
if [ -d /var/local/${HACKNAME} ] ; then
    # Be a wee bit cautious, in case something goes alwfully wrong...
    if [ -f /var/local/${HACKNAME}/env ] ; then
        logmsg "I" "uninstall" "" "removing browser workaround file"
        rm -rf /var/local/${HACKNAME}/env
    fi
    # Only remove the directory if it's really empty...
    if [ "x$( ls -A /var/local/${HACKNAME} 2> /dev/null )" == x  ] ; then
        logmsg "I" "uninstall" "" "removing empty browser workaround directory"
        rm -rf /var/local/${HACKNAME}
    fi
fi

otautils_update_progressbar

# From v3.7.N
# Remove custom directory in userstore?
logmsg "I" "uninstall" "" "removing kual extension"
if [ -d /mnt/us/extensions/${HACKNAME} ] ; then
    rm -rf /mnt/us/extensions/${HACKNAME}
    logmsg "I" "uninstall" "" "kual extension has been removed"
fi
logmsg "I" "uninstall" "" "removing custom directory (only if /mnt/us/${HACKNAME}/uninstall exists)"
if [ -d /mnt/us/${HACKNAME} -a -f /mnt/us/${HACKNAME}/uninstall ] ; then
    rm -rf /mnt/us/${HACKNAME}
    logmsg "I" "uninstall" "" "custom directory has been removed"
fi

otautils_update_progressbar

logmsg "I" "uninstall" "" "done"

otautils_update_progressbar

return 0
